/*
* Circle Follow - only mirrors what moves
* Monica Monin
* Use blobDetection and track the largest blob of a colour
* Default colour is white, click with the mouse to select a new colour to track
* Press space to adjust the camera settings, you might want to modify to get a good input
* Press c to toggle the camera visual
* Press + or - to change the threshold
* April 2009
*
*/

//Import the JMyron library
import JMyron.*;


JMyron m;//a camera object
boolean showCamera = true; //Will use this boolean variable to turn the visual of the camera on and off
boolean showBlobs = true; // Will use this boolen variable to turn the visual of the blobs on and off
//How sensitive is the colour tracking, 255 seems to work well in a dark room w/ a white light
//You would want to lower it if you have ambient lighting as there is less distinction between colours
//So play around with this value
int threshold = 255;
//These will store the position of our circle
float xPos;
float yPos;
//These will store the current position of our largest blob
float cxPos;
float cyPos;

PImage img;
 
void setup(){
  size(320,240);
  m = new JMyron();//make a new instance of the JMyron object
  m.start(width,height); //start a capture at the width and height of the display window
  m.trackColor(255,255,255,threshold);//R, G, B, and range of similarity, here we are tracking white
  m.minDensity(100); //minimum pixels in the glob required to result in a box
  m.maxDensity(10000); //maximum pixels in the glob required to result in a box
  println("Myron " + m.version());
  noFill();
  smooth();
  
  img = loadImage("win.jpg");
  // animation1 = new Animation("win_f", 14);
}

void draw(){
  
  float q = xPos;
  float w = yPos;
  
  m.update();//update the camera view
  //We use showCamera to toggle the camera on and off
  //showCamera toggles between true and false when you hit space
  if (showCamera) {
    drawCamera();//draw the camera to the screen
  } else { //showCamera must be false
    background(0); //draw the background
  }
  
  //These two arrays are multi-dimensional arrays, each index holds more than one value
  int[][] b = m.globBoxes();//get the center boxes, globBoxes returns an array containing an x,y and width and height at each index
  int[][] gc = m.globCenters(); //get the center point of the boxes, returns a point at each index

  
  float biggestBlob = 0; //this will store the area of our biggest blob
  int d = 0;//this will store the index of our biggest blob in the globBoxes array

  //here we cycle through b which is storing information on each blob to find the biggest blob, I start at
  //an index of 1 as it is drawing a blob the entire size of the screen at index 0  

  for(int i=1;i<b.length;i++){    
    //If showBlobs is true show each colour blob is it detecting, they will come up green
   if (showBlobs) {
     noFill();
     stroke(0,255,0);
     rect( b[i][0] , b[i][1] , b[i][2] , b[i][3] );
   }
    
    //Position 2 and 3 hold the width and height of the glob at index i, times them together to get the area of the blob
    float blobSize = b[i][2]*b[i][3];

    //If the current blob is bigger, then make it the biggest blob
    if (blobSize > biggestBlob) {
      biggestBlob = blobSize;
      d = i; //store its position in the array
    }
    
  }

  
  //Don't capture the position if there is no blob detected
  if (d!=0) {  //get the current position of our biggest blob, the array gc is storing the center points of each blob

    cxPos = gc[d][0];
    cyPos = gc[d][1];
    
    //Draw the biggest blob, it will come up red
    if (showBlobs) {
      stroke(255,0,0);
      noFill();
      rect( b[d][0] , b[d][1] , b[d][2] , b[d][3] );
    }
  }
  
  //So it doesn't jump around work out how far our circle is from the largest blob
  //Then move it a bit closer, here we are moving it 10% closer each time
    float distx = xPos - cxPos;
    float disty = yPos - cyPos;
    xPos -= distx / 10;
    yPos -= disty / 10;
  
  //draw our circle
   fill(255);
   noStroke();
   ellipse (xPos, yPos,30,30);
   
  //draw hole
   stroke(255);
   strokeWeight(2);
   fill(0);
   ellipse(width-width/10, height-height/10, 32, 32);
  
  //in the hole!
   if (q > width-width/10 && w > height-height/10) {
     image(img, 0, 0);
     println("You Win!");
     noLoop();
   }
 
 }

void drawCamera(){
  int[] img = m.image(); //get the normal image of the camera
  loadPixels();
  for(int i=0;i<width*height;i++){ //loop through all the pixels
    pixels[i] = img[i]; //draw each pixel to the screen
  }
  updatePixels(); 

}


void keyPressed(){
  if (key == ' ') {
  m.settings();//click the window to get the settings for the camera
  } else if (key == 'c') {
    //toggle the camera
    if (showCamera == true) {
      showCamera = false;
    } else { //showCamera must be false so make it true
      showCamera=true;
    }
  } else if (key=='+') {
    threshold+=5;
    println(threshold);
    m.sensitivity(threshold);
  } else if (key=='-') {
    threshold-=5;
    println(threshold);
    m.sensitivity(threshold);
  } else if (key == 'b') {
    //toggle the camera
    if (showBlobs == true) {
      showBlobs = false;
    } else { //showCamera must be false so make it true
      showBlobs=true;
    }
  }
  
  
  
}

void mousePressed(){
  //Pick a pixel and get its r,g,b values and track them
  int mloc = mouseX + (mouseY*width);
  color chooseColor = pixels[mloc]; 
  int r = int(red(chooseColor));
  int g = int(green(chooseColor));
  int b = int(blue(chooseColor));
  println("R:" + r + " G:" + g + " B:" + b);
  m.trackColor(r,g,b,threshold);
}


public void stop(){
  m.stop();//stop the object
  super.stop();
}

